package org.enhydra.wlh.business;

import java.io.File;
import java.io.FileFilter;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.enhydra.shark.api.RootException;
import org.enhydra.shark.utilities.MiscUtilities;
import org.enhydra.shark.utilities.SequencedHashMap;
import org.enhydra.shark.xpdl.XMLInterfaceForJDK13;
import org.enhydra.shark.xpdl.XMLUtil;

/**
 * @author Nenad Stefanovic, Sasa Bojanic
 */
public class RepositoryManager {
   private static PackageFileFilter packageFileFilter = new PackageFileFilter();

   private static RepositoryManager anInstance;

   private static String EXTERNAL_PACKAGES_REPOSITORY = "repository/external";

   private RepositoryManager() {
   }

   public static RepositoryManager getInstance() {
      if (null == anInstance) {
         synchronized (RepositoryManager.class) {
            if (null == anInstance) {
               anInstance = new RepositoryManager();
               try {
                  anInstance.setPathToXPDLRepositoryFolder(EXTERNAL_PACKAGES_REPOSITORY);
               } catch (Exception ex) {
               }
            }
         }
      }
      return anInstance;
   }

   public void setPathToXPDLRepositoryFolder(String xpdlRepFolder) throws Exception {
      RepositoryManager.EXTERNAL_PACKAGES_REPOSITORY = xpdlRepFolder;
      File f = new File(xpdlRepFolder);
      if (!f.isAbsolute()) {
         f = f.getAbsoluteFile();
      }

      if (!f.exists()) {
         throw new Exception("Folder " + xpdlRepFolder + " does not exist");
      }
      try {
         EXTERNAL_PACKAGES_REPOSITORY = f.getCanonicalPath();
      } catch (Exception ex) {
         EXTERNAL_PACKAGES_REPOSITORY = f.getAbsolutePath();
      }
   }

   public String getPathToXPDLRepositoryFolder() throws Exception {
      return EXTERNAL_PACKAGES_REPOSITORY;
   }

   public void deletePackage(String relativePath) throws Exception {
      String path = EXTERNAL_PACKAGES_REPOSITORY + File.separator + relativePath;
      File pkgFile = new File(path);
      if (!pkgFile.exists() || pkgFile.isDirectory()) {
         System.err.println("RepositoryManager -> deleting of the file "
                            + path
                            + " failed because it does not exist, or it is a directory");
         throw new Exception("RepositoryManager -> deleting of the file "
                             + path
                             + " failed because it does not exist, or it is a directory");
      }
      boolean isDeleted = false;
      try {
         isDeleted = pkgFile.delete();
      } catch (Exception ex) {
         System.err.println("RepositoryManager -> deleting of the file "
                            + path + " failed - it might be locked");
         throw new Exception("RepositoryManager -> deleting of the file "
                             + path + " failed - it might be locked");
      }
      if (!isDeleted) {
         throw new Exception("RepositoryManager -> deleting of the file "
                             + path + " NOT deleted");
      }
      System.out.println("RepositoryManager -> file "
                         + path + " is successfully deleted from repository");

   }

   public void uploadPackage(byte[] pkgContent, String relativePath) throws Exception {
      String path = EXTERNAL_PACKAGES_REPOSITORY + File.separator + relativePath;
      File pkgFile = new File(path);
      File parentFile = pkgFile.getParentFile();
      // if the file exists, or the parent file exists and it is not directory -> can't
      // create
      if (pkgFile.exists() || (parentFile.exists() && !parentFile.isDirectory())) {
         System.err.println("RepositoryManager -> upload of the file "
                            + path
                            + " failed because it already exists or its parent is not directory");
         throw new Exception("RepositoryManager -> upload of the file "
                             + path
                             + " failed because it already exists or its parent is not directory");
      }

      if (!parentFile.exists()) {
         try {
            parentFile.mkdirs();
         } catch (Exception ex) {
            throw new RootException(ex);
         }
      }

      try {
         FileOutputStream fos = new FileOutputStream(pkgFile);
         fos.write(pkgContent);
         // Write to file
         fos.flush();
         fos.close();
      } catch (Exception ex) {
         System.err.println("RepositoryManager -> upload of the file " + path + " failed");
         throw new RootException(ex);
      }

      System.out.println("RepositoryManager -> file "
                         + path + " is successfully uploaded into repository");
   }

   public String[] getPackagePaths() throws Exception {
      List packageFiles = getDefinedPackageFiles(EXTERNAL_PACKAGES_REPOSITORY, true);
      Collection pfls = new ArrayList();
      Iterator pfi = packageFiles.iterator();
      Collections.sort(packageFiles);
      while (pfi.hasNext()) {
         File f = (File) pfi.next();
         String fileName;
         try {
            fileName = f.getCanonicalPath();
         } catch (Exception ex) {
            fileName = f.getAbsolutePath();
         }
         fileName = fileName.substring(EXTERNAL_PACKAGES_REPOSITORY.length() + 1);
         pfls.add(fileName);
      }
      String[] pfs = new String[pfls.size()];
      pfls.toArray(pfs);
      return pfs;
   }

   public Map getPackagePathToIdMapping() throws Exception {
      List packageFiles = getDefinedPackageFiles(EXTERNAL_PACKAGES_REPOSITORY, true);
      Map m = new SequencedHashMap();
      Iterator pfi = packageFiles.iterator();
      Collections.sort(packageFiles);
      while (pfi.hasNext()) {
         File f = (File) pfi.next();
         String fileName;
         try {
            fileName = f.getCanonicalPath();
         } catch (Exception ex) {
            fileName = f.getAbsolutePath();
         }
         String pkgId = getIdFromFile(fileName);
         if (pkgId != null && pkgId.length() > 0) {
            fileName = fileName.substring(EXTERNAL_PACKAGES_REPOSITORY.length() + 1);
            m.put(fileName, pkgId);
         }
      }
      return m;
   }

   public String getPackageId(String relativePath) throws Exception {
      relativePath = XMLUtil.convertToSystemPath(relativePath);
      List packageFiles = getDefinedPackageFiles(EXTERNAL_PACKAGES_REPOSITORY, true);
      Iterator pfi = packageFiles.iterator();
      while (pfi.hasNext()) {
         File f = (File) pfi.next();
         String fileName;
         try {
            fileName = f.getCanonicalPath();
         } catch (Exception ex) {
            fileName = f.getAbsolutePath();
         }
         String relFileName = fileName.substring(EXTERNAL_PACKAGES_REPOSITORY.length() + 1);
         if (relFileName.equals(relativePath)) {
            return getIdFromFile(fileName);
         }
      }
      return null;
   }

   protected String getIdFromFile(String fileName) {
      // this method will be moved to XMLUtil
      return new XMLInterfaceForJDK13().getIdFromFile(fileName);
   }

   static List getDefinedPackageFiles(String repository, boolean traverse) {
      File startingFolder = new File(repository);
      List packageFiles = new ArrayList();
      if (!startingFolder.exists()) {
         System.out.println("RepositoryManager -> Repository "
                            + startingFolder + " doesn't exist");
      }
      if (traverse) {
         MiscUtilities.traverse(startingFolder, packageFiles, null);
      } else {
         packageFiles = Arrays.asList(startingFolder.listFiles(packageFileFilter));
      }

      return packageFiles;
   }

}

class PackageFileFilter implements FileFilter {
   public boolean accept(File pathname) {
      return !pathname.isDirectory();
   }
}
